# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from openai import OpenAI, AsyncOpenAI
from tests.utils import assert_matches_type
from openai.pagination import SyncCursorPage, AsyncCursorPage
from openai.types.beta.threads import (
    Run,
)

# pyright: reportDeprecated=false

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestRuns:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create_overload_1(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            run = client.beta.threads.runs.create(
                thread_id="thread_id",
                assistant_id="assistant_id",
            )

        assert_matches_type(Run, run, path=["response"])

    @parametrize
    def test_method_create_with_all_params_overload_1(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            run = client.beta.threads.runs.create(
                thread_id="thread_id",
                assistant_id="assistant_id",
                include=["step_details.tool_calls[*].file_search.results[*].content"],
                additional_instructions="additional_instructions",
                additional_messages=[
                    {
                        "content": "string",
                        "role": "user",
                        "attachments": [
                            {
                                "file_id": "file_id",
                                "tools": [{"type": "code_interpreter"}],
                            }
                        ],
                        "metadata": {"foo": "string"},
                    }
                ],
                instructions="instructions",
                max_completion_tokens=256,
                max_prompt_tokens=256,
                metadata={"foo": "string"},
                model="string",
                parallel_tool_calls=True,
                reasoning_effort="low",
                response_format="auto",
                stream=False,
                temperature=1,
                tool_choice="none",
                tools=[{"type": "code_interpreter"}],
                top_p=1,
                truncation_strategy={
                    "type": "auto",
                    "last_messages": 1,
                },
            )

        assert_matches_type(Run, run, path=["response"])

    @parametrize
    def test_raw_response_create_overload_1(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.beta.threads.runs.with_raw_response.create(
                thread_id="thread_id",
                assistant_id="assistant_id",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        run = response.parse()
        assert_matches_type(Run, run, path=["response"])

    @parametrize
    def test_streaming_response_create_overload_1(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with client.beta.threads.runs.with_streaming_response.create(
                thread_id="thread_id",
                assistant_id="assistant_id",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                run = response.parse()
                assert_matches_type(Run, run, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_create_overload_1(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
                client.beta.threads.runs.with_raw_response.create(
                    thread_id="",
                    assistant_id="assistant_id",
                )

    @parametrize
    def test_method_create_overload_2(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            run_stream = client.beta.threads.runs.create(
                thread_id="thread_id",
                assistant_id="assistant_id",
                stream=True,
            )

        run_stream.response.close()

    @parametrize
    def test_method_create_with_all_params_overload_2(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            run_stream = client.beta.threads.runs.create(
                thread_id="thread_id",
                assistant_id="assistant_id",
                stream=True,
                include=["step_details.tool_calls[*].file_search.results[*].content"],
                additional_instructions="additional_instructions",
                additional_messages=[
                    {
                        "content": "string",
                        "role": "user",
                        "attachments": [
                            {
                                "file_id": "file_id",
                                "tools": [{"type": "code_interpreter"}],
                            }
                        ],
                        "metadata": {"foo": "string"},
                    }
                ],
                instructions="instructions",
                max_completion_tokens=256,
                max_prompt_tokens=256,
                metadata={"foo": "string"},
                model="string",
                parallel_tool_calls=True,
                reasoning_effort="low",
                response_format="auto",
                temperature=1,
                tool_choice="none",
                tools=[{"type": "code_interpreter"}],
                top_p=1,
                truncation_strategy={
                    "type": "auto",
                    "last_messages": 1,
                },
            )

        run_stream.response.close()

    @parametrize
    def test_raw_response_create_overload_2(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.beta.threads.runs.with_raw_response.create(
                thread_id="thread_id",
                assistant_id="assistant_id",
                stream=True,
            )

        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        stream = response.parse()
        stream.close()

    @parametrize
    def test_streaming_response_create_overload_2(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with client.beta.threads.runs.with_streaming_response.create(
                thread_id="thread_id",
                assistant_id="assistant_id",
                stream=True,
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                stream = response.parse()
                stream.close()

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_create_overload_2(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
                client.beta.threads.runs.with_raw_response.create(
                    thread_id="",
                    assistant_id="assistant_id",
                    stream=True,
                )

    @parametrize
    def test_method_retrieve(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            run = client.beta.threads.runs.retrieve(
                run_id="run_id",
                thread_id="thread_id",
            )

        assert_matches_type(Run, run, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.beta.threads.runs.with_raw_response.retrieve(
                run_id="run_id",
                thread_id="thread_id",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        run = response.parse()
        assert_matches_type(Run, run, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with client.beta.threads.runs.with_streaming_response.retrieve(
                run_id="run_id",
                thread_id="thread_id",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                run = response.parse()
                assert_matches_type(Run, run, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
                client.beta.threads.runs.with_raw_response.retrieve(
                    run_id="run_id",
                    thread_id="",
                )

            with pytest.raises(ValueError, match=r"Expected a non-empty value for `run_id` but received ''"):
                client.beta.threads.runs.with_raw_response.retrieve(
                    run_id="",
                    thread_id="thread_id",
                )

    @parametrize
    def test_method_update(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            run = client.beta.threads.runs.update(
                run_id="run_id",
                thread_id="thread_id",
            )

        assert_matches_type(Run, run, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            run = client.beta.threads.runs.update(
                run_id="run_id",
                thread_id="thread_id",
                metadata={"foo": "string"},
            )

        assert_matches_type(Run, run, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.beta.threads.runs.with_raw_response.update(
                run_id="run_id",
                thread_id="thread_id",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        run = response.parse()
        assert_matches_type(Run, run, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with client.beta.threads.runs.with_streaming_response.update(
                run_id="run_id",
                thread_id="thread_id",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                run = response.parse()
                assert_matches_type(Run, run, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
                client.beta.threads.runs.with_raw_response.update(
                    run_id="run_id",
                    thread_id="",
                )

            with pytest.raises(ValueError, match=r"Expected a non-empty value for `run_id` but received ''"):
                client.beta.threads.runs.with_raw_response.update(
                    run_id="",
                    thread_id="thread_id",
                )

    @parametrize
    def test_method_list(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            run = client.beta.threads.runs.list(
                thread_id="thread_id",
            )

        assert_matches_type(SyncCursorPage[Run], run, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            run = client.beta.threads.runs.list(
                thread_id="thread_id",
                after="after",
                before="before",
                limit=0,
                order="asc",
            )

        assert_matches_type(SyncCursorPage[Run], run, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.beta.threads.runs.with_raw_response.list(
                thread_id="thread_id",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        run = response.parse()
        assert_matches_type(SyncCursorPage[Run], run, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with client.beta.threads.runs.with_streaming_response.list(
                thread_id="thread_id",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                run = response.parse()
                assert_matches_type(SyncCursorPage[Run], run, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
                client.beta.threads.runs.with_raw_response.list(
                    thread_id="",
                )

    @parametrize
    def test_method_cancel(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            run = client.beta.threads.runs.cancel(
                run_id="run_id",
                thread_id="thread_id",
            )

        assert_matches_type(Run, run, path=["response"])

    @parametrize
    def test_raw_response_cancel(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.beta.threads.runs.with_raw_response.cancel(
                run_id="run_id",
                thread_id="thread_id",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        run = response.parse()
        assert_matches_type(Run, run, path=["response"])

    @parametrize
    def test_streaming_response_cancel(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with client.beta.threads.runs.with_streaming_response.cancel(
                run_id="run_id",
                thread_id="thread_id",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                run = response.parse()
                assert_matches_type(Run, run, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_cancel(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
                client.beta.threads.runs.with_raw_response.cancel(
                    run_id="run_id",
                    thread_id="",
                )

            with pytest.raises(ValueError, match=r"Expected a non-empty value for `run_id` but received ''"):
                client.beta.threads.runs.with_raw_response.cancel(
                    run_id="",
                    thread_id="thread_id",
                )

    @parametrize
    def test_method_submit_tool_outputs_overload_1(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            run = client.beta.threads.runs.submit_tool_outputs(
                run_id="run_id",
                thread_id="thread_id",
                tool_outputs=[{}],
            )

        assert_matches_type(Run, run, path=["response"])

    @parametrize
    def test_method_submit_tool_outputs_with_all_params_overload_1(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            run = client.beta.threads.runs.submit_tool_outputs(
                run_id="run_id",
                thread_id="thread_id",
                tool_outputs=[
                    {
                        "output": "output",
                        "tool_call_id": "tool_call_id",
                    }
                ],
                stream=False,
            )

        assert_matches_type(Run, run, path=["response"])

    @parametrize
    def test_raw_response_submit_tool_outputs_overload_1(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.beta.threads.runs.with_raw_response.submit_tool_outputs(
                run_id="run_id",
                thread_id="thread_id",
                tool_outputs=[{}],
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        run = response.parse()
        assert_matches_type(Run, run, path=["response"])

    @parametrize
    def test_streaming_response_submit_tool_outputs_overload_1(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with client.beta.threads.runs.with_streaming_response.submit_tool_outputs(
                run_id="run_id",
                thread_id="thread_id",
                tool_outputs=[{}],
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                run = response.parse()
                assert_matches_type(Run, run, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_submit_tool_outputs_overload_1(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
                client.beta.threads.runs.with_raw_response.submit_tool_outputs(
                    run_id="run_id",
                    thread_id="",
                    tool_outputs=[{}],
                )

            with pytest.raises(ValueError, match=r"Expected a non-empty value for `run_id` but received ''"):
                client.beta.threads.runs.with_raw_response.submit_tool_outputs(
                    run_id="",
                    thread_id="thread_id",
                    tool_outputs=[{}],
                )

    @parametrize
    def test_method_submit_tool_outputs_overload_2(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            run_stream = client.beta.threads.runs.submit_tool_outputs(
                run_id="run_id",
                thread_id="thread_id",
                stream=True,
                tool_outputs=[{}],
            )

        run_stream.response.close()

    @parametrize
    def test_raw_response_submit_tool_outputs_overload_2(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.beta.threads.runs.with_raw_response.submit_tool_outputs(
                run_id="run_id",
                thread_id="thread_id",
                stream=True,
                tool_outputs=[{}],
            )

        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        stream = response.parse()
        stream.close()

    @parametrize
    def test_streaming_response_submit_tool_outputs_overload_2(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with client.beta.threads.runs.with_streaming_response.submit_tool_outputs(
                run_id="run_id",
                thread_id="thread_id",
                stream=True,
                tool_outputs=[{}],
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                stream = response.parse()
                stream.close()

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_submit_tool_outputs_overload_2(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
                client.beta.threads.runs.with_raw_response.submit_tool_outputs(
                    run_id="run_id",
                    thread_id="",
                    stream=True,
                    tool_outputs=[{}],
                )

            with pytest.raises(ValueError, match=r"Expected a non-empty value for `run_id` but received ''"):
                client.beta.threads.runs.with_raw_response.submit_tool_outputs(
                    run_id="",
                    thread_id="thread_id",
                    stream=True,
                    tool_outputs=[{}],
                )


class TestAsyncRuns:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create_overload_1(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            run = await async_client.beta.threads.runs.create(
                thread_id="thread_id",
                assistant_id="assistant_id",
            )

        assert_matches_type(Run, run, path=["response"])

    @parametrize
    async def test_method_create_with_all_params_overload_1(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            run = await async_client.beta.threads.runs.create(
                thread_id="thread_id",
                assistant_id="assistant_id",
                include=["step_details.tool_calls[*].file_search.results[*].content"],
                additional_instructions="additional_instructions",
                additional_messages=[
                    {
                        "content": "string",
                        "role": "user",
                        "attachments": [
                            {
                                "file_id": "file_id",
                                "tools": [{"type": "code_interpreter"}],
                            }
                        ],
                        "metadata": {"foo": "string"},
                    }
                ],
                instructions="instructions",
                max_completion_tokens=256,
                max_prompt_tokens=256,
                metadata={"foo": "string"},
                model="string",
                parallel_tool_calls=True,
                reasoning_effort="low",
                response_format="auto",
                stream=False,
                temperature=1,
                tool_choice="none",
                tools=[{"type": "code_interpreter"}],
                top_p=1,
                truncation_strategy={
                    "type": "auto",
                    "last_messages": 1,
                },
            )

        assert_matches_type(Run, run, path=["response"])

    @parametrize
    async def test_raw_response_create_overload_1(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.beta.threads.runs.with_raw_response.create(
                thread_id="thread_id",
                assistant_id="assistant_id",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        run = response.parse()
        assert_matches_type(Run, run, path=["response"])

    @parametrize
    async def test_streaming_response_create_overload_1(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.beta.threads.runs.with_streaming_response.create(
                thread_id="thread_id",
                assistant_id="assistant_id",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                run = await response.parse()
                assert_matches_type(Run, run, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_create_overload_1(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
                await async_client.beta.threads.runs.with_raw_response.create(
                    thread_id="",
                    assistant_id="assistant_id",
                )

    @parametrize
    async def test_method_create_overload_2(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            run_stream = await async_client.beta.threads.runs.create(
                thread_id="thread_id",
                assistant_id="assistant_id",
                stream=True,
            )

        await run_stream.response.aclose()

    @parametrize
    async def test_method_create_with_all_params_overload_2(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            run_stream = await async_client.beta.threads.runs.create(
                thread_id="thread_id",
                assistant_id="assistant_id",
                stream=True,
                include=["step_details.tool_calls[*].file_search.results[*].content"],
                additional_instructions="additional_instructions",
                additional_messages=[
                    {
                        "content": "string",
                        "role": "user",
                        "attachments": [
                            {
                                "file_id": "file_id",
                                "tools": [{"type": "code_interpreter"}],
                            }
                        ],
                        "metadata": {"foo": "string"},
                    }
                ],
                instructions="instructions",
                max_completion_tokens=256,
                max_prompt_tokens=256,
                metadata={"foo": "string"},
                model="string",
                parallel_tool_calls=True,
                reasoning_effort="low",
                response_format="auto",
                temperature=1,
                tool_choice="none",
                tools=[{"type": "code_interpreter"}],
                top_p=1,
                truncation_strategy={
                    "type": "auto",
                    "last_messages": 1,
                },
            )

        await run_stream.response.aclose()

    @parametrize
    async def test_raw_response_create_overload_2(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.beta.threads.runs.with_raw_response.create(
                thread_id="thread_id",
                assistant_id="assistant_id",
                stream=True,
            )

        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        stream = response.parse()
        await stream.close()

    @parametrize
    async def test_streaming_response_create_overload_2(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.beta.threads.runs.with_streaming_response.create(
                thread_id="thread_id",
                assistant_id="assistant_id",
                stream=True,
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                stream = await response.parse()
                await stream.close()

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_create_overload_2(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
                await async_client.beta.threads.runs.with_raw_response.create(
                    thread_id="",
                    assistant_id="assistant_id",
                    stream=True,
                )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            run = await async_client.beta.threads.runs.retrieve(
                run_id="run_id",
                thread_id="thread_id",
            )

        assert_matches_type(Run, run, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.beta.threads.runs.with_raw_response.retrieve(
                run_id="run_id",
                thread_id="thread_id",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        run = response.parse()
        assert_matches_type(Run, run, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.beta.threads.runs.with_streaming_response.retrieve(
                run_id="run_id",
                thread_id="thread_id",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                run = await response.parse()
                assert_matches_type(Run, run, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
                await async_client.beta.threads.runs.with_raw_response.retrieve(
                    run_id="run_id",
                    thread_id="",
                )

            with pytest.raises(ValueError, match=r"Expected a non-empty value for `run_id` but received ''"):
                await async_client.beta.threads.runs.with_raw_response.retrieve(
                    run_id="",
                    thread_id="thread_id",
                )

    @parametrize
    async def test_method_update(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            run = await async_client.beta.threads.runs.update(
                run_id="run_id",
                thread_id="thread_id",
            )

        assert_matches_type(Run, run, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            run = await async_client.beta.threads.runs.update(
                run_id="run_id",
                thread_id="thread_id",
                metadata={"foo": "string"},
            )

        assert_matches_type(Run, run, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.beta.threads.runs.with_raw_response.update(
                run_id="run_id",
                thread_id="thread_id",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        run = response.parse()
        assert_matches_type(Run, run, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.beta.threads.runs.with_streaming_response.update(
                run_id="run_id",
                thread_id="thread_id",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                run = await response.parse()
                assert_matches_type(Run, run, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
                await async_client.beta.threads.runs.with_raw_response.update(
                    run_id="run_id",
                    thread_id="",
                )

            with pytest.raises(ValueError, match=r"Expected a non-empty value for `run_id` but received ''"):
                await async_client.beta.threads.runs.with_raw_response.update(
                    run_id="",
                    thread_id="thread_id",
                )

    @parametrize
    async def test_method_list(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            run = await async_client.beta.threads.runs.list(
                thread_id="thread_id",
            )

        assert_matches_type(AsyncCursorPage[Run], run, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            run = await async_client.beta.threads.runs.list(
                thread_id="thread_id",
                after="after",
                before="before",
                limit=0,
                order="asc",
            )

        assert_matches_type(AsyncCursorPage[Run], run, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.beta.threads.runs.with_raw_response.list(
                thread_id="thread_id",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        run = response.parse()
        assert_matches_type(AsyncCursorPage[Run], run, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.beta.threads.runs.with_streaming_response.list(
                thread_id="thread_id",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                run = await response.parse()
                assert_matches_type(AsyncCursorPage[Run], run, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
                await async_client.beta.threads.runs.with_raw_response.list(
                    thread_id="",
                )

    @parametrize
    async def test_method_cancel(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            run = await async_client.beta.threads.runs.cancel(
                run_id="run_id",
                thread_id="thread_id",
            )

        assert_matches_type(Run, run, path=["response"])

    @parametrize
    async def test_raw_response_cancel(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.beta.threads.runs.with_raw_response.cancel(
                run_id="run_id",
                thread_id="thread_id",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        run = response.parse()
        assert_matches_type(Run, run, path=["response"])

    @parametrize
    async def test_streaming_response_cancel(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.beta.threads.runs.with_streaming_response.cancel(
                run_id="run_id",
                thread_id="thread_id",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                run = await response.parse()
                assert_matches_type(Run, run, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_cancel(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
                await async_client.beta.threads.runs.with_raw_response.cancel(
                    run_id="run_id",
                    thread_id="",
                )

            with pytest.raises(ValueError, match=r"Expected a non-empty value for `run_id` but received ''"):
                await async_client.beta.threads.runs.with_raw_response.cancel(
                    run_id="",
                    thread_id="thread_id",
                )

    @parametrize
    async def test_method_submit_tool_outputs_overload_1(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            run = await async_client.beta.threads.runs.submit_tool_outputs(
                run_id="run_id",
                thread_id="thread_id",
                tool_outputs=[{}],
            )

        assert_matches_type(Run, run, path=["response"])

    @parametrize
    async def test_method_submit_tool_outputs_with_all_params_overload_1(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            run = await async_client.beta.threads.runs.submit_tool_outputs(
                run_id="run_id",
                thread_id="thread_id",
                tool_outputs=[
                    {
                        "output": "output",
                        "tool_call_id": "tool_call_id",
                    }
                ],
                stream=False,
            )

        assert_matches_type(Run, run, path=["response"])

    @parametrize
    async def test_raw_response_submit_tool_outputs_overload_1(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.beta.threads.runs.with_raw_response.submit_tool_outputs(
                run_id="run_id",
                thread_id="thread_id",
                tool_outputs=[{}],
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        run = response.parse()
        assert_matches_type(Run, run, path=["response"])

    @parametrize
    async def test_streaming_response_submit_tool_outputs_overload_1(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.beta.threads.runs.with_streaming_response.submit_tool_outputs(
                run_id="run_id",
                thread_id="thread_id",
                tool_outputs=[{}],
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                run = await response.parse()
                assert_matches_type(Run, run, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_submit_tool_outputs_overload_1(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
                await async_client.beta.threads.runs.with_raw_response.submit_tool_outputs(
                    run_id="run_id",
                    thread_id="",
                    tool_outputs=[{}],
                )

            with pytest.raises(ValueError, match=r"Expected a non-empty value for `run_id` but received ''"):
                await async_client.beta.threads.runs.with_raw_response.submit_tool_outputs(
                    run_id="",
                    thread_id="thread_id",
                    tool_outputs=[{}],
                )

    @parametrize
    async def test_method_submit_tool_outputs_overload_2(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            run_stream = await async_client.beta.threads.runs.submit_tool_outputs(
                run_id="run_id",
                thread_id="thread_id",
                stream=True,
                tool_outputs=[{}],
            )

        await run_stream.response.aclose()

    @parametrize
    async def test_raw_response_submit_tool_outputs_overload_2(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.beta.threads.runs.with_raw_response.submit_tool_outputs(
                run_id="run_id",
                thread_id="thread_id",
                stream=True,
                tool_outputs=[{}],
            )

        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        stream = response.parse()
        await stream.close()

    @parametrize
    async def test_streaming_response_submit_tool_outputs_overload_2(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.beta.threads.runs.with_streaming_response.submit_tool_outputs(
                run_id="run_id",
                thread_id="thread_id",
                stream=True,
                tool_outputs=[{}],
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                stream = await response.parse()
                await stream.close()

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_submit_tool_outputs_overload_2(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
                await async_client.beta.threads.runs.with_raw_response.submit_tool_outputs(
                    run_id="run_id",
                    thread_id="",
                    stream=True,
                    tool_outputs=[{}],
                )

            with pytest.raises(ValueError, match=r"Expected a non-empty value for `run_id` but received ''"):
                await async_client.beta.threads.runs.with_raw_response.submit_tool_outputs(
                    run_id="",
                    thread_id="thread_id",
                    stream=True,
                    tool_outputs=[{}],
                )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from openai import OpenAI, AsyncOpenAI
from tests.utils import assert_matches_type
from openai.pagination import SyncPage, AsyncPage, SyncCursorPage, AsyncCursorPage
from openai.types.fine_tuning.checkpoints import (
    PermissionCreateResponse,
    PermissionDeleteResponse,
    PermissionRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPermissions:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: OpenAI) -> None:
        permission = client.fine_tuning.checkpoints.permissions.create(
            fine_tuned_model_checkpoint="ft:gpt-4o-mini-2024-07-18:org:weather:B7R9VjQd",
            project_ids=["string"],
        )
        assert_matches_type(SyncPage[PermissionCreateResponse], permission, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: OpenAI) -> None:
        response = client.fine_tuning.checkpoints.permissions.with_raw_response.create(
            fine_tuned_model_checkpoint="ft:gpt-4o-mini-2024-07-18:org:weather:B7R9VjQd",
            project_ids=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        permission = response.parse()
        assert_matches_type(SyncPage[PermissionCreateResponse], permission, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: OpenAI) -> None:
        with client.fine_tuning.checkpoints.permissions.with_streaming_response.create(
            fine_tuned_model_checkpoint="ft:gpt-4o-mini-2024-07-18:org:weather:B7R9VjQd",
            project_ids=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            permission = response.parse()
            assert_matches_type(SyncPage[PermissionCreateResponse], permission, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_create(self, client: OpenAI) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `fine_tuned_model_checkpoint` but received ''"
        ):
            client.fine_tuning.checkpoints.permissions.with_raw_response.create(
                fine_tuned_model_checkpoint="",
                project_ids=["string"],
            )

    @parametrize
    def test_method_retrieve(self, client: OpenAI) -> None:
        permission = client.fine_tuning.checkpoints.permissions.retrieve(
            fine_tuned_model_checkpoint="ft-AF1WoRqd3aJAHsqc9NY7iL8F",
        )
        assert_matches_type(SyncCursorPage[PermissionRetrieveResponse], permission, path=["response"])

    @parametrize
    def test_method_retrieve_with_all_params(self, client: OpenAI) -> None:
        permission = client.fine_tuning.checkpoints.permissions.retrieve(
            fine_tuned_model_checkpoint="ft-AF1WoRqd3aJAHsqc9NY7iL8F",
            after="after",
            limit=0,
            order="ascending",
            project_id="project_id",
        )
        assert_matches_type(SyncCursorPage[PermissionRetrieveResponse], permission, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: OpenAI) -> None:
        response = client.fine_tuning.checkpoints.permissions.with_raw_response.retrieve(
            fine_tuned_model_checkpoint="ft-AF1WoRqd3aJAHsqc9NY7iL8F",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        permission = response.parse()
        assert_matches_type(SyncCursorPage[PermissionRetrieveResponse], permission, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: OpenAI) -> None:
        with client.fine_tuning.checkpoints.permissions.with_streaming_response.retrieve(
            fine_tuned_model_checkpoint="ft-AF1WoRqd3aJAHsqc9NY7iL8F",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            permission = response.parse()
            assert_matches_type(SyncCursorPage[PermissionRetrieveResponse], permission, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: OpenAI) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `fine_tuned_model_checkpoint` but received ''"
        ):
            client.fine_tuning.checkpoints.permissions.with_raw_response.retrieve(
                fine_tuned_model_checkpoint="",
            )

    @parametrize
    def test_method_delete(self, client: OpenAI) -> None:
        permission = client.fine_tuning.checkpoints.permissions.delete(
            permission_id="cp_zc4Q7MP6XxulcVzj4MZdwsAB",
            fine_tuned_model_checkpoint="ft:gpt-4o-mini-2024-07-18:org:weather:B7R9VjQd",
        )
        assert_matches_type(PermissionDeleteResponse, permission, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: OpenAI) -> None:
        response = client.fine_tuning.checkpoints.permissions.with_raw_response.delete(
            permission_id="cp_zc4Q7MP6XxulcVzj4MZdwsAB",
            fine_tuned_model_checkpoint="ft:gpt-4o-mini-2024-07-18:org:weather:B7R9VjQd",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        permission = response.parse()
        assert_matches_type(PermissionDeleteResponse, permission, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: OpenAI) -> None:
        with client.fine_tuning.checkpoints.permissions.with_streaming_response.delete(
            permission_id="cp_zc4Q7MP6XxulcVzj4MZdwsAB",
            fine_tuned_model_checkpoint="ft:gpt-4o-mini-2024-07-18:org:weather:B7R9VjQd",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            permission = response.parse()
            assert_matches_type(PermissionDeleteResponse, permission, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: OpenAI) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `fine_tuned_model_checkpoint` but received ''"
        ):
            client.fine_tuning.checkpoints.permissions.with_raw_response.delete(
                permission_id="cp_zc4Q7MP6XxulcVzj4MZdwsAB",
                fine_tuned_model_checkpoint="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `permission_id` but received ''"):
            client.fine_tuning.checkpoints.permissions.with_raw_response.delete(
                permission_id="",
                fine_tuned_model_checkpoint="ft:gpt-4o-mini-2024-07-18:org:weather:B7R9VjQd",
            )


class TestAsyncPermissions:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncOpenAI) -> None:
        permission = await async_client.fine_tuning.checkpoints.permissions.create(
            fine_tuned_model_checkpoint="ft:gpt-4o-mini-2024-07-18:org:weather:B7R9VjQd",
            project_ids=["string"],
        )
        assert_matches_type(AsyncPage[PermissionCreateResponse], permission, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.fine_tuning.checkpoints.permissions.with_raw_response.create(
            fine_tuned_model_checkpoint="ft:gpt-4o-mini-2024-07-18:org:weather:B7R9VjQd",
            project_ids=["string"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        permission = response.parse()
        assert_matches_type(AsyncPage[PermissionCreateResponse], permission, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncOpenAI) -> None:
        async with async_client.fine_tuning.checkpoints.permissions.with_streaming_response.create(
            fine_tuned_model_checkpoint="ft:gpt-4o-mini-2024-07-18:org:weather:B7R9VjQd",
            project_ids=["string"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            permission = await response.parse()
            assert_matches_type(AsyncPage[PermissionCreateResponse], permission, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_create(self, async_client: AsyncOpenAI) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `fine_tuned_model_checkpoint` but received ''"
        ):
            await async_client.fine_tuning.checkpoints.permissions.with_raw_response.create(
                fine_tuned_model_checkpoint="",
                project_ids=["string"],
            )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncOpenAI) -> None:
        permission = await async_client.fine_tuning.checkpoints.permissions.retrieve(
            fine_tuned_model_checkpoint="ft-AF1WoRqd3aJAHsqc9NY7iL8F",
        )
        assert_matches_type(AsyncCursorPage[PermissionRetrieveResponse], permission, path=["response"])

    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncOpenAI) -> None:
        permission = await async_client.fine_tuning.checkpoints.permissions.retrieve(
            fine_tuned_model_checkpoint="ft-AF1WoRqd3aJAHsqc9NY7iL8F",
            after="after",
            limit=0,
            order="ascending",
            project_id="project_id",
        )
        assert_matches_type(AsyncCursorPage[PermissionRetrieveResponse], permission, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.fine_tuning.checkpoints.permissions.with_raw_response.retrieve(
            fine_tuned_model_checkpoint="ft-AF1WoRqd3aJAHsqc9NY7iL8F",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        permission = response.parse()
        assert_matches_type(AsyncCursorPage[PermissionRetrieveResponse], permission, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncOpenAI) -> None:
        async with async_client.fine_tuning.checkpoints.permissions.with_streaming_response.retrieve(
            fine_tuned_model_checkpoint="ft-AF1WoRqd3aJAHsqc9NY7iL8F",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            permission = await response.parse()
            assert_matches_type(AsyncCursorPage[PermissionRetrieveResponse], permission, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncOpenAI) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `fine_tuned_model_checkpoint` but received ''"
        ):
            await async_client.fine_tuning.checkpoints.permissions.with_raw_response.retrieve(
                fine_tuned_model_checkpoint="",
            )

    @parametrize
    async def test_method_delete(self, async_client: AsyncOpenAI) -> None:
        permission = await async_client.fine_tuning.checkpoints.permissions.delete(
            permission_id="cp_zc4Q7MP6XxulcVzj4MZdwsAB",
            fine_tuned_model_checkpoint="ft:gpt-4o-mini-2024-07-18:org:weather:B7R9VjQd",
        )
        assert_matches_type(PermissionDeleteResponse, permission, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.fine_tuning.checkpoints.permissions.with_raw_response.delete(
            permission_id="cp_zc4Q7MP6XxulcVzj4MZdwsAB",
            fine_tuned_model_checkpoint="ft:gpt-4o-mini-2024-07-18:org:weather:B7R9VjQd",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        permission = response.parse()
        assert_matches_type(PermissionDeleteResponse, permission, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncOpenAI) -> None:
        async with async_client.fine_tuning.checkpoints.permissions.with_streaming_response.delete(
            permission_id="cp_zc4Q7MP6XxulcVzj4MZdwsAB",
            fine_tuned_model_checkpoint="ft:gpt-4o-mini-2024-07-18:org:weather:B7R9VjQd",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            permission = await response.parse()
            assert_matches_type(PermissionDeleteResponse, permission, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncOpenAI) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `fine_tuned_model_checkpoint` but received ''"
        ):
            await async_client.fine_tuning.checkpoints.permissions.with_raw_response.delete(
                permission_id="cp_zc4Q7MP6XxulcVzj4MZdwsAB",
                fine_tuned_model_checkpoint="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `permission_id` but received ''"):
            await async_client.fine_tuning.checkpoints.permissions.with_raw_response.delete(
                permission_id="",
                fine_tuned_model_checkpoint="ft:gpt-4o-mini-2024-07-18:org:weather:B7R9VjQd",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["InputAudioBufferSpeechStoppedEvent"]


class InputAudioBufferSpeechStoppedEvent(BaseModel):
    audio_end_ms: int
    """Milliseconds since the session started when speech stopped.

    This will correspond to the end of audio sent to the model, and thus includes
    the `min_silence_duration_ms` configured in the Session.
    """

    event_id: str
    """The unique ID of the server event."""

    item_id: str
    """The ID of the user message item that will be created."""

    type: Literal["input_audio_buffer.speech_stopped"]
    """The event type, must be `input_audio_buffer.speech_stopped`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from .response import Response
from ..._models import BaseModel

__all__ = ["ResponseQueuedEvent"]


class ResponseQueuedEvent(BaseModel):
    response: Response
    """The full response object that is queued."""

    sequence_number: int
    """The sequence number for this event."""

    type: Literal["response.queued"]
    """The type of the event. Always 'response.queued'."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Optional
from typing_extensions import Literal, Required, TypedDict

from .._types import FileTypes
from .image_model import ImageModel

__all__ = ["ImageEditParams"]


class ImageEditParams(TypedDict, total=False):
    image: Required[Union[FileTypes, List[FileTypes]]]
    """The image(s) to edit. Must be a supported image file or an array of images.

    For `gpt-image-1`, each image should be a `png`, `webp`, or `jpg` file less than
    50MB. You can provide up to 16 images.

    For `dall-e-2`, you can only provide one image, and it should be a square `png`
    file less than 4MB.
    """

    prompt: Required[str]
    """A text description of the desired image(s).

    The maximum length is 1000 characters for `dall-e-2`, and 32000 characters for
    `gpt-image-1`.
    """

    background: Optional[Literal["transparent", "opaque", "auto"]]
    """Allows to set transparency for the background of the generated image(s).

    This parameter is only supported for `gpt-image-1`. Must be one of
    `transparent`, `opaque` or `auto` (default value). When `auto` is used, the
    model will automatically determine the best background for the image.

    If `transparent`, the output format needs to support transparency, so it should
    be set to either `png` (default value) or `webp`.
    """

    mask: FileTypes
    """An additional image whose fully transparent areas (e.g.

    where alpha is zero) indicate where `image` should be edited. If there are
    multiple images provided, the mask will be applied on the first image. Must be a
    valid PNG file, less than 4MB, and have the same dimensions as `image`.
    """

    model: Union[str, ImageModel, None]
    """The model to use for image generation.

    Only `dall-e-2` and `gpt-image-1` are supported. Defaults to `dall-e-2` unless a
    parameter specific to `gpt-image-1` is used.
    """

    n: Optional[int]
    """The number of images to generate. Must be between 1 and 10."""

    output_compression: Optional[int]
    """The compression level (0-100%) for the generated images.

    This parameter is only supported for `gpt-image-1` with the `webp` or `jpeg`
    output formats, and defaults to 100.
    """

    output_format: Optional[Literal["png", "jpeg", "webp"]]
    """The format in which the generated images are returned.

    This parameter is only supported for `gpt-image-1`. Must be one of `png`,
    `jpeg`, or `webp`. The default value is `png`.
    """

    quality: Optional[Literal["standard", "low", "medium", "high", "auto"]]
    """The quality of the image that will be generated.

    `high`, `medium` and `low` are only supported for `gpt-image-1`. `dall-e-2` only
    supports `standard` quality. Defaults to `auto`.
    """

    response_format: Optional[Literal["url", "b64_json"]]
    """The format in which the generated images are returned.

    Must be one of `url` or `b64_json`. URLs are only valid for 60 minutes after the
    image has been generated. This parameter is only supported for `dall-e-2`, as
    `gpt-image-1` will always return base64-encoded images.
    """

    size: Optional[Literal["256x256", "512x512", "1024x1024", "1536x1024", "1024x1536", "auto"]]
    """The size of the generated images.

    Must be one of `1024x1024`, `1536x1024` (landscape), `1024x1536` (portrait), or
    `auto` (default value) for `gpt-image-1`, and one of `256x256`, `512x512`, or
    `1024x1024` for `dall-e-2`.
    """

    user: str
    """
    A unique identifier representing your end-user, which can help OpenAI to monitor
    and detect abuse.
    [Learn more](https://platform.openai.com/docs/guides/safety-best-practices#end-user-ids).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from .dpo_hyperparameters import DpoHyperparameters

__all__ = ["DpoMethod"]


class DpoMethod(BaseModel):
    hyperparameters: Optional[DpoHyperparameters] = None
    """The hyperparameters used for the DPO fine-tuning job."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ....._models import BaseModel

__all__ = ["FunctionToolCall", "Function"]


class Function(BaseModel):
    arguments: str
    """The arguments passed to the function."""

    name: str
    """The name of the function."""

    output: Optional[str] = None
    """The output of the function.

    This will be `null` if the outputs have not been
    [submitted](https://platform.openai.com/docs/api-reference/runs/submitToolOutputs)
    yet.
    """


class FunctionToolCall(BaseModel):
    id: str
    """The ID of the tool call object."""

    function: Function
    """The definition of the function that was called."""

    type: Literal["function"]
    """The type of tool call.

    This is always going to be `function` for this type of tool call.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from openai import OpenAI, AsyncOpenAI
from tests.utils import assert_matches_type
from openai.pagination import SyncCursorPage, AsyncCursorPage
from openai.types.beta.threads import (
    Message,
    MessageDeleted,
)

# pyright: reportDeprecated=false

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestMessages:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            message = client.beta.threads.messages.create(
                thread_id="thread_id",
                content="string",
                role="user",
            )

        assert_matches_type(Message, message, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            message = client.beta.threads.messages.create(
                thread_id="thread_id",
                content="string",
                role="user",
                attachments=[
                    {
                        "file_id": "file_id",
                        "tools": [{"type": "code_interpreter"}],
                    }
                ],
                metadata={"foo": "string"},
            )

        assert_matches_type(Message, message, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.beta.threads.messages.with_raw_response.create(
                thread_id="thread_id",
                content="string",
                role="user",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = response.parse()
        assert_matches_type(Message, message, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with client.beta.threads.messages.with_streaming_response.create(
                thread_id="thread_id",
                content="string",
                role="user",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                message = response.parse()
                assert_matches_type(Message, message, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_create(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
                client.beta.threads.messages.with_raw_response.create(
                    thread_id="",
                    content="string",
                    role="user",
                )

    @parametrize
    def test_method_retrieve(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            message = client.beta.threads.messages.retrieve(
                message_id="message_id",
                thread_id="thread_id",
            )

        assert_matches_type(Message, message, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.beta.threads.messages.with_raw_response.retrieve(
                message_id="message_id",
                thread_id="thread_id",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = response.parse()
        assert_matches_type(Message, message, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with client.beta.threads.messages.with_streaming_response.retrieve(
                message_id="message_id",
                thread_id="thread_id",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                message = response.parse()
                assert_matches_type(Message, message, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
                client.beta.threads.messages.with_raw_response.retrieve(
                    message_id="message_id",
                    thread_id="",
                )

            with pytest.raises(ValueError, match=r"Expected a non-empty value for `message_id` but received ''"):
                client.beta.threads.messages.with_raw_response.retrieve(
                    message_id="",
                    thread_id="thread_id",
                )

    @parametrize
    def test_method_update(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            message = client.beta.threads.messages.update(
                message_id="message_id",
                thread_id="thread_id",
            )

        assert_matches_type(Message, message, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            message = client.beta.threads.messages.update(
                message_id="message_id",
                thread_id="thread_id",
                metadata={"foo": "string"},
            )

        assert_matches_type(Message, message, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.beta.threads.messages.with_raw_response.update(
                message_id="message_id",
                thread_id="thread_id",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = response.parse()
        assert_matches_type(Message, message, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with client.beta.threads.messages.with_streaming_response.update(
                message_id="message_id",
                thread_id="thread_id",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                message = response.parse()
                assert_matches_type(Message, message, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
                client.beta.threads.messages.with_raw_response.update(
                    message_id="message_id",
                    thread_id="",
                )

            with pytest.raises(ValueError, match=r"Expected a non-empty value for `message_id` but received ''"):
                client.beta.threads.messages.with_raw_response.update(
                    message_id="",
                    thread_id="thread_id",
                )

    @parametrize
    def test_method_list(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            message = client.beta.threads.messages.list(
                thread_id="thread_id",
            )

        assert_matches_type(SyncCursorPage[Message], message, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            message = client.beta.threads.messages.list(
                thread_id="thread_id",
                after="after",
                before="before",
                limit=0,
                order="asc",
                run_id="run_id",
            )

        assert_matches_type(SyncCursorPage[Message], message, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.beta.threads.messages.with_raw_response.list(
                thread_id="thread_id",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = response.parse()
        assert_matches_type(SyncCursorPage[Message], message, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with client.beta.threads.messages.with_streaming_response.list(
                thread_id="thread_id",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                message = response.parse()
                assert_matches_type(SyncCursorPage[Message], message, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
                client.beta.threads.messages.with_raw_response.list(
                    thread_id="",
                )

    @parametrize
    def test_method_delete(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            message = client.beta.threads.messages.delete(
                message_id="message_id",
                thread_id="thread_id",
            )

        assert_matches_type(MessageDeleted, message, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.beta.threads.messages.with_raw_response.delete(
                message_id="message_id",
                thread_id="thread_id",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = response.parse()
        assert_matches_type(MessageDeleted, message, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with client.beta.threads.messages.with_streaming_response.delete(
                message_id="message_id",
                thread_id="thread_id",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                message = response.parse()
                assert_matches_type(MessageDeleted, message, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
                client.beta.threads.messages.with_raw_response.delete(
                    message_id="message_id",
                    thread_id="",
                )

            with pytest.raises(ValueError, match=r"Expected a non-empty value for `message_id` but received ''"):
                client.beta.threads.messages.with_raw_response.delete(
                    message_id="",
                    thread_id="thread_id",
                )


class TestAsyncMessages:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            message = await async_client.beta.threads.messages.create(
                thread_id="thread_id",
                content="string",
                role="user",
            )

        assert_matches_type(Message, message, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            message = await async_client.beta.threads.messages.create(
                thread_id="thread_id",
                content="string",
                role="user",
                attachments=[
                    {
                        "file_id": "file_id",
                        "tools": [{"type": "code_interpreter"}],
                    }
                ],
                metadata={"foo": "string"},
            )

        assert_matches_type(Message, message, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.beta.threads.messages.with_raw_response.create(
                thread_id="thread_id",
                content="string",
                role="user",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = response.parse()
        assert_matches_type(Message, message, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.beta.threads.messages.with_streaming_response.create(
                thread_id="thread_id",
                content="string",
                role="user",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                message = await response.parse()
                assert_matches_type(Message, message, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_create(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
                await async_client.beta.threads.messages.with_raw_response.create(
                    thread_id="",
                    content="string",
                    role="user",
                )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            message = await async_client.beta.threads.messages.retrieve(
                message_id="message_id",
                thread_id="thread_id",
            )

        assert_matches_type(Message, message, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.beta.threads.messages.with_raw_response.retrieve(
                message_id="message_id",
                thread_id="thread_id",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = response.parse()
        assert_matches_type(Message, message, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.beta.threads.messages.with_streaming_response.retrieve(
                message_id="message_id",
                thread_id="thread_id",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                message = await response.parse()
                assert_matches_type(Message, message, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
                await async_client.beta.threads.messages.with_raw_response.retrieve(
                    message_id="message_id",
                    thread_id="",
                )

            with pytest.raises(ValueError, match=r"Expected a non-empty value for `message_id` but received ''"):
                await async_client.beta.threads.messages.with_raw_response.retrieve(
                    message_id="",
                    thread_id="thread_id",
                )

    @parametrize
    async def test_method_update(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            message = await async_client.beta.threads.messages.update(
                message_id="message_id",
                thread_id="thread_id",
            )

        assert_matches_type(Message, message, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            message = await async_client.beta.threads.messages.update(
                message_id="message_id",
                thread_id="thread_id",
                metadata={"foo": "string"},
            )

        assert_matches_type(Message, message, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.beta.threads.messages.with_raw_response.update(
                message_id="message_id",
                thread_id="thread_id",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = response.parse()
        assert_matches_type(Message, message, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.beta.threads.messages.with_streaming_response.update(
                message_id="message_id",
                thread_id="thread_id",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                message = await response.parse()
                assert_matches_type(Message, message, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
                await async_client.beta.threads.messages.with_raw_response.update(
                    message_id="message_id",
                    thread_id="",
                )

            with pytest.raises(ValueError, match=r"Expected a non-empty value for `message_id` but received ''"):
                await async_client.beta.threads.messages.with_raw_response.update(
                    message_id="",
                    thread_id="thread_id",
                )

    @parametrize
    async def test_method_list(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            message = await async_client.beta.threads.messages.list(
                thread_id="thread_id",
            )

        assert_matches_type(AsyncCursorPage[Message], message, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            message = await async_client.beta.threads.messages.list(
                thread_id="thread_id",
                after="after",
                before="before",
                limit=0,
                order="asc",
                run_id="run_id",
            )

        assert_matches_type(AsyncCursorPage[Message], message, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.beta.threads.messages.with_raw_response.list(
                thread_id="thread_id",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = response.parse()
        assert_matches_type(AsyncCursorPage[Message], message, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.beta.threads.messages.with_streaming_response.list(
                thread_id="thread_id",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                message = await response.parse()
                assert_matches_type(AsyncCursorPage[Message], message, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
                await async_client.beta.threads.messages.with_raw_response.list(
                    thread_id="",
                )

    @parametrize
    async def test_method_delete(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            message = await async_client.beta.threads.messages.delete(
                message_id="message_id",
                thread_id="thread_id",
            )

        assert_matches_type(MessageDeleted, message, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.beta.threads.messages.with_raw_response.delete(
                message_id="message_id",
                thread_id="thread_id",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = response.parse()
        assert_matches_type(MessageDeleted, message, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.beta.threads.messages.with_streaming_response.delete(
                message_id="message_id",
                thread_id="thread_id",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                message = await response.parse()
                assert_matches_type(MessageDeleted, message, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
                await async_client.beta.threads.messages.with_raw_response.delete(
                    message_id="message_id",
                    thread_id="",
                )

            with pytest.raises(ValueError, match=r"Expected a non-empty value for `message_id` but received ''"):
                await async_client.beta.threads.messages.with_raw_response.delete(
                    message_id="",
                    thread_id="thread_id",
                )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Literal, Required, TypeAlias, TypedDict

__all__ = ["ResponseCodeInterpreterToolCallParam", "Result", "ResultLogs", "ResultFiles", "ResultFilesFile"]


class ResultLogs(TypedDict, total=False):
    logs: Required[str]
    """The logs of the code interpreter tool call."""

    type: Required[Literal["logs"]]
    """The type of the code interpreter text output. Always `logs`."""


class ResultFilesFile(TypedDict, total=False):
    file_id: Required[str]
    """The ID of the file."""

    mime_type: Required[str]
    """The MIME type of the file."""


class ResultFiles(TypedDict, total=False):
    files: Required[Iterable[ResultFilesFile]]

    type: Required[Literal["files"]]
    """The type of the code interpreter file output. Always `files`."""


Result: TypeAlias = Union[ResultLogs, ResultFiles]


class ResponseCodeInterpreterToolCallParam(TypedDict, total=False):
    id: Required[str]
    """The unique ID of the code interpreter tool call."""

    code: Required[str]
    """The code to run."""

    results: Required[Iterable[Result]]
    """The results of the code interpreter tool call."""

    status: Required[Literal["in_progress", "interpreting", "completed"]]
    """The status of the code interpreter tool call."""

    type: Required[Literal["code_interpreter_call"]]
    """The type of the code interpreter tool call. Always `code_interpreter_call`."""

    container_id: str
    """The ID of the container used to run the code."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["ResponseFunctionCallArgumentsDeltaEvent"]


class ResponseFunctionCallArgumentsDeltaEvent(BaseModel):
    delta: str
    """The function-call arguments delta that is added."""

    item_id: str
    """The ID of the output item that the function-call arguments delta is added to."""

    output_index: int
    """
    The index of the output item that the function-call arguments delta is added to.
    """

    sequence_number: int
    """The sequence number of this event."""

    type: Literal["response.function_call_arguments.delta"]
    """The type of the event. Always `response.function_call_arguments.delta`."""
